﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.Text.RegularExpressions;
using Microsoft.Win32;


namespace EPT_System_Demo
{
    public partial class EPT_System_Demo
    {

        //Parameters
        public byte COMMAND_DECODE = 0x38;
        public const int AH_CS_UNRELIABLE_DROP = 0;                       // Flag of dropped data
        public const int AH_CS_UNRELIABLE_FLUSH = 1;

        //------------------------------------------------
        // Active Host Communications Parameters
        public const byte TRIGGER_IN_COMMAND = 0x08;
        public const byte TRANSFER_IN_COMMAND = 0x10;
        public const byte BLOCK_IN_COMMAND = 0x20;

        public const byte TRIGGER_OUT_COMMAND = 0x08;
        public const byte BYTE_TRANSFER_OUT_COMMAND = 0x10;
        public const byte BLOCK_TRANSFER_OUT_COMMAND = 0x20;

        public const byte OUT_CONTROL_WORD = 0x80;
        public const byte IN_CONTROL_WORD = 0x80;

        // Active Control Register Parameters
        public const byte TRANSFER_CONTROL_BYTE1 = 0x5A;
        public const byte TRANSFER_CONTROL_BYTE2 = 0xC3;
        public const byte TRANSFER_CONTROL_BYTE3 = 0x7E;
        public const byte TRANSFER_CONTROL_DUMMY_BYTE = 0x00;

        //Create an array of the Transfer Class for device
        Receive EPTReceiveDevice = new Receive();

        //Start of Class Methods

        List<string> ComPortNames(String VID, String PID)
        {
            String pattern = String.Format("^VID_{0}.PID_{1}", VID, PID);
            Regex _rx = new Regex(pattern, RegexOptions.IgnoreCase);
            List<string> comports = new List<string>();
            RegistryKey rk1 = Registry.LocalMachine;
            RegistryKey rk2 = rk1.OpenSubKey("SYSTEM\\CurrentControlSet\\Enum");
            foreach (String s3 in rk2.GetSubKeyNames())
            {
                RegistryKey rk3 = rk2.OpenSubKey(s3);
                foreach (String s in rk3.GetSubKeyNames())
                {
                    if (_rx.Match(s).Success)
                    {
                        RegistryKey rk4 = rk3.OpenSubKey(s);
                        foreach (String s2 in rk4.GetSubKeyNames())
                        {
                            RegistryKey rk5 = rk4.OpenSubKey(s2);
                            comports.Add((string)rk5.GetValue("Device Parameters"));
                            RegistryKey rk6 = rk5.OpenSubKey("Device Parameters");
                            comports.Add((string)rk6.GetValue("PortName"));
                        }
                    }
                }
            }
            return comports;
        }
        //------------------------------------------------
        // The Following Section of Visual Studio Code Sets 
        // up the Serial Port, manages the Port and closes
        // the Serial Port.
        //------------------------------------------------

        //Serial Port Management
        public void CloseSerialPort1()
        {
            //Check if serialPort1 is open, then close the serial port and window
            if (FT220xSerialPort.IsOpen)
            {
                FT220xSerialPort.Close();
            }
            btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;
            //textBox1.ReadOnly = true;

        }

        public bool OpenSerialPort1()
        {
            try
            {
                //Set the serial port parameters
                FT220xSerialPort.PortName = PortName;
                FT220xSerialPort.BaudRate = Convert.ToInt32(BaudRate);
                FT220xSerialPort.Parity = (Parity)Enum.Parse(typeof(Parity), vParity);
                FT220xSerialPort.DataBits = Convert.ToInt16(DataBits);
                FT220xSerialPort.StopBits = (StopBits)Enum.Parse(typeof(StopBits), StopBits);
                FT220xSerialPort.Handshake = (Handshake)Enum.Parse(typeof(Handshake), pHandshake);

                if (!FT220xSerialPort.IsOpen)
                {
                    FT220xSerialPort.Open();
                    btnOpenDevice.Enabled = false;
                    btnCloseDevice.Enabled = true;
                    //textBox1.ReadOnly = false;
                    return true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            return false;
        }


        //------------------------------------------------
        // The Following Section of Visual Studio Code Receives
        // the stream from the Serial Port. The Serial Port 
        // stream is decoded and messages determined. The messages
        // are parsed and a call back function is called with 
        // message details
        //------------------------------------------------


        private void FT220xSerialPort_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            uint c;
            uint eptBytesToRead=0;

            if (FT220xSerialPort.BytesToRead < 2)
            {
                return;
            }
            int bytes = FT220xSerialPort.BytesToRead;
            int bytesStored = 0;
            byte[] buffer = new byte[bytes];
            FT220xSerialPort.Read(buffer, 0, bytes);

            //Check for incoming block 
            c = (uint)buffer[0];
            c = c & 0xf8;
            if (c == 0xe0)
            {
                bytesStored = bytesStored + bytes;
                eptBytesToRead = (uint)buffer[1];
                byte[] blockBuffer = new byte[eptBytesToRead + 2];
                System.Buffer.BlockCopy(buffer, 0, blockBuffer, 0, bytes);
                while (bytesStored <= eptBytesToRead)
                {
                    bytes = FT220xSerialPort.BytesToRead;
                    buffer = new byte[bytes];
                    FT220xSerialPort.Read(buffer, 0, bytes);

                    System.Buffer.BlockCopy(buffer, 0, blockBuffer, bytesStored, buffer.Length);
                    bytesStored = bytesStored + bytes;
                }

                this.Invoke(new MethodInvoker(delegate () { EPT_AH_Receive(blockBuffer); }));

            }
            else
            {
                //string WriteRcvChar = "";
                // WriteRcvChar = String.Format("{0}", (int)buffer[0]);
                this.Invoke(new MethodInvoker(delegate () { EPT_AH_Receive(buffer); }));

            }
        }

        public void EPT_AH_Receive(byte[] receiveBytes)
        {
            uint c, a, p, l, index;
            //Compare first byte to the incoming message code
            string s = String.Empty;
            foreach (byte b in receiveBytes)
            {
                s += String.Format("{0:x2}", (int)System.Convert.ToUInt32(b.ToString()));
                s += "\r\n";
            }
            //tbBlockRcv.AppendText(s);
            //this.Invoke(new MethodInvoker(delegate () { textBox1.AppendText(s); }));


            //Write the command into the EPTReceiveDevice 
            c = (uint)receiveBytes[0];
            c = c & 0xf8;

            //Write the address into EPTReceiveDevice
            a = (uint)receiveBytes[0];
            a = a & 0x07;
            EPTReceiveDevice.Address = a;
            //Display Address to text box
            string r = String.Empty;
            r += String.Format("EPTReceiveDevice Address= {0:x2}", EPTReceiveDevice.Address);
            r += "\r\n";
            //this.Invoke(new MethodInvoker(delegate () { textBox1.AppendText(r); }));

            switch (c)
            {
                case 0xc8:
                    //this.Invoke(new MethodInvoker(delegate () { textBox1.AppendText("Trigger Recieved\r\n"); }));
                    EPTReceiveDevice.Command = TRIGGER_IN_COMMAND;
                    break;
                case 0xd0:
                    //this.Invoke(new MethodInvoker(delegate () { textBox1.AppendText("Transfer Byte Recieved\r\n"); }));
                    EPTReceiveDevice.Command = TRANSFER_IN_COMMAND;
                    break;
                case 0xe0:
                    //this.Invoke(new MethodInvoker(delegate () { textBox1.AppendText("Block Recieved\r\n"); }));
                    EPTReceiveDevice.Command = BLOCK_IN_COMMAND;

                    l = (uint)receiveBytes[1];
                    EPTReceiveDevice.Length = l;
                    //Display Length to text box
                    string n = String.Empty;
                    n += String.Format("EPTReceiveDevice Length = {0:x2}", EPTReceiveDevice.Length);
                    n += "\r\n";

                    EPTReceiveDevice.cBlockBuf = new byte[EPTReceiveDevice.Length];
                    //this.Invoke(new MethodInvoker(delegate () { textBox1.AppendText(n); }));
                    for (index = 2; index <= EPTReceiveDevice.Length+1; index++)
                    {
                        EPTReceiveDevice.cBlockBuf[index - 2] = receiveBytes[index];
                    }
                    //Once EPTReceiveDevice has been populated, call the Receive Parser function
                    EPTParseReceive();
                    return;
                //break;
                default:
                    //this.textBox1.AppendText(text);
                    break;

            }


            //Write the payload into EPTReceiveDevice
            p = (uint)receiveBytes[1];
            EPTReceiveDevice.Payload = p;
            //Display Payload to text box
            string q = String.Empty;
            q += String.Format("EPTReceiveDevice Payload = {0:x2}", EPTReceiveDevice.Payload);
            q += "\r\n";

            //this.Invoke(new MethodInvoker(delegate () { textBox1.AppendText(q); }));

            //Once EPTReceiveDevice has been populated, call the Receive Parser function
            EPTParseReceive();
        }

        //------------------------------------------------
        // The Following Section of Visual Studio Code sends 
        // Triggers, Bytes and Block data to the EPT Device
        //------------------------------------------------
        private void EPT_AH_SendTrigger(byte trigger_value)
        {
            PutBuff = new byte[1];


            // Load element 0 with the command.
            PutBuff[0] = (byte)(OUT_CONTROL_WORD + TRIGGER_OUT_COMMAND + 0x00);
            //Send byte to COM Port
            FT220xSerialPort.Write(PutBuff, 0, 1);

            // Load element 0 with the payload.
            PutBuff[0] = (byte)trigger_value;
            //Send byte to COM Port
            FT220xSerialPort.Write(PutBuff, 0, 1);
        }

        private void EPT_AH_SendByte(byte device_channel, byte data_byte)
        {
            //byte tempVar = 0;
            PutBuff = new byte[1];

            // Load element 0 with the command.
            PutBuff[0] = (byte)(OUT_CONTROL_WORD + BYTE_TRANSFER_OUT_COMMAND + device_channel);
            //Send byte to COM Port
            FT220xSerialPort.Write(PutBuff, 0, 1);

            // Load element 0 with the payload.
            PutBuff[0] = (byte)data_byte;
            //Send byte to COM Port
            FT220xSerialPort.Write(PutBuff, 0, 1);
        }

        private void EPT_AH_SendBlock(byte device_channel, byte data_length, byte[] data_array)
        {
            PutBuff = new byte[1];

            // Load element 0 with the command.
            PutBuff[0] = (byte)(OUT_CONTROL_WORD + BLOCK_TRANSFER_OUT_COMMAND + device_channel);
            //Send byte to COM Port
            FT220xSerialPort.Write(PutBuff, 0, 1);

            // Load element 0 with the length of data to follow in block out.
            PutBuff[0] = (byte)data_length;
            //Send byte to COM Port
            FT220xSerialPort.Write(PutBuff, 0, 1);

            foreach (byte b in data_array)
            {

                // Load element 0 with the payload.
                PutBuff[0] = b;
                //Send byte to COM Port
                FT220xSerialPort.Write(PutBuff, 0, 1);

            }
        }

        private void EPT_AH_SendTransferControlByte(byte address_to_device, byte payload)
        {
            //Send first byte in Control Register Sequence
            EPT_AH_SendByte((byte)address_to_device, TRANSFER_CONTROL_BYTE1);
            Thread.Sleep(1);

            //Send second byte in Control Register Sequence
            EPT_AH_SendByte((byte)address_to_device, TRANSFER_CONTROL_BYTE2);
            Thread.Sleep(1);

            //Send third byte in Control Register Sequence
            EPT_AH_SendByte((byte)address_to_device, TRANSFER_CONTROL_BYTE3);
            Thread.Sleep(1);

            //Send data byte
            EPT_AH_SendByte((byte)address_to_device, (byte)payload);
            Thread.Sleep(1);

            //Send dummy byte in Control Register Sequence
            //EPT_AH_SendByte((byte)address_to_device, TRANSFER_CONTROL_DUMMY_BYTE);
            //Thread.Sleep(1);


        }






    }

    // Transfer object definition
    public class Receive
    {
        public uint Command;
        public uint Address;
        public uint Length;
        public uint Payload;
        public byte[] cBlockBuf;
        public bool TransferPending;
        public uint Repititions;

        public Receive()
        {
            Command = 0;
            Address = 0;
            Length = 0;
            Payload = 0;
            TransferPending = false;
            Repititions = 0;
        }

    }

}
