//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2022
//#
//#
//# File Name:  EPT_10CL016_AF_T2_Top.v
//# Author:     Earth People Technology
//# Date:       November 13, 2022
//# Revision:   A
//#
//# Development: EPT Platform Demonstration Project 
//# Application: Altera Cyclone 10 FPGA
//# Description: This file contains verilog code which will allow access
//#              to Active Transfer Library. 
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			11/13/22 	1			Created
//#
//#									
//#
//#######################################################################


//************************************************************************
//* Module Declaration
//************************************************************************

module EPT_10CL016_AF_T2_Top (

	
	input   wire                CLK_50MHZ,
	input   wire                RST_N,
	
    //SPI Communications FT-220X
	output  wire                SPI_SCLK,
	input   wire                SPI_MISO,
	inout   wire                SPI_MIOSIO_1,
	output  wire                SPI_SS,

	//User LED control Registers
	output reg   [7:0]          USER_LEDS,
	
    //User GPIOs
	input   wire                UB49,//XIO_1[1] -- UB49 -- J11 Pin 2
	input   wire                UB21,//XIO_2[7] -- UB21 -- J9 Pin 3
	input   wire                UB36,//XIO_2[6] -- UB36 -- J9 Pin 2
	input   wire                UB37,//XIO_2[5] -- UB37 -- J9 Pin 1
	
	input   wire [9:0]          UB6,//XIO_4[9:0] -- UB69 to UB60 -- J13 Pins 10 to Pin 1  
	input   wire [3:0]          UB1,//XIO_6[3:0] -- UB13 to UB10 -- J8 Pins 1 to Pin 4  
	
	input   wire                UB19,//XIO_5[6] -- UB19 -- J6 Pin 7
	input   wire                UB20,//XIO_5[7] -- UB20 -- J6 Pin 8
	input   wire                UB9,//XIO_8[5] -- UB9 -- J12 Pin 6
	input   wire                UB8,//XIO_6[4] -- UB8 -- J8 Pin 5
	input   wire                UB7,//XIO_6[5] -- UB7 -- J8 Pin 6
	input   wire                UB40,//XIO_6[7] -- UB40 -- J8 Pin 8
	input   wire                UB41,//XIO_7[7] -- UB41 -- J7 Pin 5
	input   wire                UB42,//XIO_7[7] -- UB42 -- J7 Pin 2
	input   wire                UB43,//XIO_7[7] -- UB43 -- J7 Pin 1
	input   wire                UB44,//XIO_8[2] -- UB44 -- J12 Pin 3
	input   wire                UB45,//XIO_8[0] -- UB45 -- J12 Pin 1
	input   wire                UB58,//XIO_8[6] -- UB58 -- J12 Pin 7
	input   wire                UB59,//XIO_8[7] -- UB59 -- J12 Pin 8
	
	//User RGB LED Control Registers
	output  wire                RGB_LED_BLUE,
	output  wire                RGB_LED_RED,
	output  wire                RGB_LED_GREEN,
	
	output  wire  [7:0]         LED_DISPLAY_1,
	output  wire  [7:0]         LED_DISPLAY_2,
	output  wire  [7:0]         LED_DISPLAY_3,
	
	input   wire                PB_SWITCH_1,
	input   wire                PB_SWITCH_2
	
	
	);

   //-----------------------------------------------
   // Parameters
   //-----------------------------------------------
   
   parameter                   GLOBAL_RESET_COUNT = 12'h09c8;

   //Transfer to Host State Machine
   parameter                   TRANSFER_HOST_IDLE = 1,
                               TRANSFER_HOST_LOOPBACK_START = 2,
							   TRANSFER_HOST_LOOPBACK_COMPLETE = 3;

   
   //HDMI Registers
   //wire                pixclk;
   //reg [9:0] CounterX, CounterY;
   //reg hSync, vSync, DrawArea;
   
   
   
   //LED State Machine Parameters
   parameter                   IDLE = 0,
	                           SELECT_MODE = 1,
                               LOAD_LEDS = 2,
                               LOAD_REGISTER = 3,
                               WAIT_FOR_TIMER = 4;
							   
   //Push Button Switches
   parameter                   PB_SWITCH_1_ASSERT    = 8'h01;
   parameter                   PB_SWITCH_2_ASSERT    = 8'h02;

   //RGB LED Assert Parameters
   parameter                   ASSERT_RGB_LED_BLUE    = 8'h01;
   parameter                   ASSERT_RGB_LED_RED     = 8'h02;
   parameter                   ASSERT_RGB_LED_GREEN   = 8'h04;
   parameter                   DEASSERT_RGB_LED       = 8'h08;
   
   // Block Transfer Options
	parameter                  BLOCK_COUNT_16       = 8'h10;

	//Block Transfer to Host State Machine
   parameter                   BLOCK_LOOPBACK_IDLE = 1,
                               BLOCK_LOOPBACK_INIT = 2,
							   FIFO_EN_HIGH = 3,
							   FIFO_EN_LOW = 4;
							   
   //Number of GPIO Storage Registers on the CycloFlex
   parameter                   CYCLO_GPIO_MAX_NUMBER = 8;

	//Block Transfer to Host Counter
   parameter                   BLK_TRANSFER_OUT_IDLE = 1,
			                   BLK_TRANSFER_OUT_COUNT = 2,
							   BLK_TRANSFER_OUT_COMPLETE = 3;

   //Control Register selections for the LED Blinky
   parameter                   RANDOM_NUMBER = 4'h1;
   parameter                   SHIFT_LEFT = 4'h2;
   parameter                   SHIFT_RIGHT = 4'h3;
   parameter                   COUNT_UP = 4'h4;
   parameter                   COUNT_DOWN = 4'h5;
   parameter                   STATIC_VALUE = 6'h6;

   //Timer Value Low Limit
`ifdef SIM
   parameter                   TIMER_LOW_LIMIT       = 24'h1f;
   parameter                   HIGH_Z_TIMER_VALUE    = 24'h8f;
`else
   parameter                   TIMER_LOW_LIMIT       = 24'h2625a0;
   parameter                   HIGH_Z_TIMER_VALUE    = 24'ha1220; //10 ms
   //parameter                   HIGH_Z_TIMER_VALUE    = 24'h325aa0; //50 ms
`endif

`ifdef SIM
   	reg [8*30:1] state_name;
`endif
   
//***************************************************************************
//* Internal Signals and Registers Declarations
//***************************************************************************

	//Finite State Machine control registers
	reg  [4:0]                 state, next;

	//Active SPI Registers
	wire [1:0]                 aa;
	wire                       RST;

	wire [23:0]                UC_IN;
	wire [21:0]                UC_OUT;
	
	
	//Clock registers
	wire                       CLK_50;
	
	//SPI Registers
	wire                       spi_mosi;
	
	//Trigger Signals
	reg  [7:0]                 trigger_out;
	wire [7:0]                 trigger_in_byte;	
	reg  [7:0]                 trigger_from_host_latch;
	
	//LED registers
	wire                       led_reset;
	
	//Switch registers
	wire                       switch_reset;
	
	//Transfer registers Address 2
	reg                        transfer_out_reg;
	wire                       transfer_in_received;
	wire [7:0]                 transfer_in_byte;
	reg  [7:0]                 transfer_out_byte;
	reg  [3:0]                 transfer_to_host_counter;
	reg  [3:0]                 transfer_to_host_state;

	
	reg                        transfer_out_led_display_start;
	wire                       transfer_in_received_led_display;
	wire [7:0]                 transfer_out_led_display;
	wire [7:0]                 transfer_in_led_display;	
	
	//Block Transfer Out registers
	reg [2:0]                  state_block_loopback;
	reg                        block_out_reg;
	wire                       block_out_received;
	reg [7:0]                  block_out_counter;
	reg                        start_block_transfer;
	reg                        start_block_transfer_reg;
	wire                       block_byte_ready;
	reg  [7:0]                 block_out_byte;
	reg [1:0]                  block_transfer_out_state;
	
	//Block Transfer In registers
	wire [7:0]                 block_in_data;
	wire                       block_in_rcv;
	reg                        block_in_loopback;
	wire  [7:0]                ept_length;
	wire  [7:0]                uc_length;
	wire                       block_busy;
	reg   [7:0]                block_in_data_counter;
	reg                        fifo_wren_reg;

    //REG LED Registers
    reg   [7:0]                rgb_led_byte;
	reg                        rgb_led_enable;
	

	//Reset signals
	wire                       reset;
	reg  [11:0]                reset_counter;
	reg                        reset_signal_reg;
	

	//LED BLinky Signals
	reg                        start_transfer_userled;
	reg                        start_transfer_userled_reg;
	reg                        transfer_out_userled_reg;
	wire                       transfer_in_received_userled;
	wire [7:0]                 transfer_in_byte_userled;
	wire [7:0]                 transfer_out_byte_userled;	
	reg  [23:0]                timer_value;	
	reg                        start_blinky;
	reg  [7:0]                 static_load_value;	
	reg  [7:0]                 shift_count_value;
	reg  [7:0]                 led_reg;
	wire                       exit_blinky;
	wire [7:0]                 control_register;
	reg  [23:0]                led_delay_counter;
	
	//Seven Segment LED Display Registers
	reg  [7:0]                 seven_segment_led_display_byte_1;
	reg                        seven_segment_led_display_enable_1;
	reg  [7:0]                 seven_segment_led_display_byte_2;
	reg                        seven_segment_led_display_enable_2;
	reg  [7:0]                 seven_segment_led_display_byte_3;
	reg                        seven_segment_led_display_enable_3;
	reg  [7:0]                 transfer_in_hold_byte;
	
	//GPIO Connectors
	wire [7:0]                 XIO_1;
	wire [7:0]                 XIO_2;
	wire [7:0]                 XIO_3;
	wire [7:0]                 XIO_4;
	wire [7:0]                 XIO_5;
	wire [7:0]                 XIO_6;
	wire [7:0]                 XIO_7;
	wire [7:0]                 XIO_8;

	
    //Test Registers	
	wire [7:0]                 active_serial_state_out;
	wire [11:0]                 active_transfer_test_out; 
	
	


//***************************************************************************
//* 	Signal Assignments	
//***************************************************************************
   
   //Clock and Reset
   //assign            aa[1] = CLK_50MHZ;
   assign            RST = RST_N;
   assign            aa = {CLK_50MHZ, RST};

   //Block to host byte
   //assign            block_out_byte = mem_array[block_out_counter];
   assign            uc_length = 8'h8;
   assign            ept_length = 8'h8;
   
   //Active Transfer 1 registers
   assign            transfer_out_byte_userled = 1'b0;
   
   //Active Transfer 3 registers
   assign            transfer_out_led_display = 1'b0;
   
    //Test Connector Mapping 

	assign           XIO_1[7] = LED_DISPLAY_1[1];// -- UB46 -- J11 Pin 8 -- U3 Pin 144
	assign           XIO_1[6] = LED_DISPLAY_1[2];// -- UB47 -- J11 Pin 7 -- U3 Pin 143
	assign           XIO_1[5] = LED_DISPLAY_1[7];// -- UB48 -- J11 Pin 6 -- U3 Pin 142
	assign           XIO_1[4] = LED_DISPLAY_1[6];// -- UB30 -- J11 Pin 5 -- U3 Pin 141
	assign           XIO_1[3] = LED_DISPLAY_1[4];// -- UB57 -- J11 Pin 4 -- U3 Pin 137
	assign           XIO_1[2] = LED_DISPLAY_1[3];// -- UB31 -- J11 Pin 3 -- U3 Pin 136
	assign           XIO_1[1] = UB49;// -- UB49 -- J11 Pin 2 -- U3 Pin 134
	assign           XIO_1[0] = LED_DISPLAY_1[5];// -- UB56 -- J11 Pin 1 -- U3 Pin 135
	
	
   assign            XIO_2[7] = LED_DISPLAY_3[3];// -- UB53 -- J9 Pin 8 -- U3 Pin 103
   assign            XIO_2[6] = LED_DISPLAY_3[4];// -- UB52 -- J9 Pin 7 -- U3 Pin  101 
   assign            XIO_2[5] = LED_DISPLAY_3[6];// -- UB35 -- J9 Pin 6 -- U3 Pin 100
   assign            XIO_2[4] = LED_DISPLAY_3[5];// -- UB51 -- J9 Pin 5 -- U3 Pin 99
   assign            XIO_2[3] = LED_DISPLAY_3[0];// -- UB50 -- J9 Pin 4 -- U3 Pin 98
   assign            XIO_2[2] = UB21;// -- UB21 -- J9 Pin 3 -- U3 Pin 85
   assign            XIO_2[1] = UB36;// -- UB36 -- J9 Pin 2 -- U3 Pin 83
   assign            XIO_2[0] = UB37;// -- UB37 -- J9 Pin 1 -- U3 Pin 80

   assign            XIO_3[0] = LED_DISPLAY_1[0]; // -- UB55 -- J10 Pin 8 -- U3 Pin 133
   assign            XIO_3[1] = LED_DISPLAY_2[2]; // -- UB54 -- J10 Pin 7 -- U3 Pin 132
   assign            XIO_3[2] = LED_DISPLAY_3[1]; // -- UB32 -- J10 Pin 6 -- U3 Pin 119   
   assign            XIO_3[3] = USER_LEDS[4];// -- UB25 -- J10 Pin 5 -- U3 Pin 114
   assign            XIO_3[4] = USER_LEDS[6];// -- UB24 -- J10 Pin 4 -- U3 Pin 113
   assign            XIO_3[5] = USER_LEDS[5];   // -- UB23 -- J10 Pin 3 -- U3 Pin 112  
   assign            XIO_3[6] = LED_DISPLAY_3[2];// -- UB33 -- J10 Pin 2 -- U3 Pin 111  
   assign            XIO_3[7] = LED_DISPLAY_3[7];// -- UB34 -- J10 Pin 1 -- U3 Pin 105 

   assign            XIO_4[0] = UB6[0];//UB60 -- J11 Pin 1 -- U3 Pin 52
   assign            XIO_4[1] = UB6[1];//UB61 -- J11 Pin 2 -- U3 Pin 53
   assign            XIO_4[2] = UB6[2];//UB62 -- J11 Pin 3 -- U3 Pin 54
   assign            XIO_4[3] = UB6[3];//UB63 -- J11 Pin 4 -- U3 Pin 55
   assign            XIO_4[4] = UB6[4];//UB64 -- J11 Pin 5 -- U3 Pin 88 
   assign            XIO_4[5] = UB6[5];//UB65 -- J11 Pin 6 -- U3 Pin 89
   assign            XIO_4[6] = UB6[6];//UB66 -- J11 Pin 7 -- U3 Pin 90
   assign            XIO_4[7] = UB6[7];//UB67 -- J11 Pin 8 -- U3 Pin 127

   assign            XIO_5[0] = LED_DISPLAY_2[0];// -- UB39 -- J6 Pin 1 -- U3 Pin 65
   assign            XIO_5[1] = LED_DISPLAY_2[1];// -- UB14 -- J6 Pin 2 -- U3 Pin 66 
   assign            XIO_5[2] = LED_DISPLAY_2[3];// -- UB15 -- J6 Pin 3 -- U3 Pin 67
   assign            XIO_5[3] = LED_DISPLAY_2[4];// -- UB16 -- J6 Pin 4 -- U3 Pin 68
   assign            XIO_5[4] = LED_DISPLAY_2[6];// -- UB17 -- J6 Pin 5 -- U3 Pin 71
   assign            XIO_5[5] = LED_DISPLAY_2[7];// -- UB18 -- J6 Pin 6 -- U3 Pin 72
   assign            XIO_5[6] = UB19;// -- UB19 -- J6 Pin 7 -- U3 Pin 76
   assign            XIO_5[7] = UB20;// -- UB20 -- J6 Pin 8 -- U3 Pin 77

    assign            XIO_6[0] = UB1[3];//UB13 -- J8 Pin 1 -- U3 Pin 61
    assign            XIO_6[1] = UB1[2];//UB12 -- J8 Pin 2 -- U3 Pin 60
    assign            XIO_6[2] = UB1[1];//UB11 -- J8 Pin 3 -- U3 Pin 59
    assign            XIO_6[3] = UB1[0];//UB10 -- J8 Pin 4 -- U3 Pin 58
    assign            XIO_6[4] = UB8;//UB8 -- J8 Pin 5 -- U3 Pin 51
    assign            XIO_6[5] = UB7;//UB7 -- J8 Pin 46 -- U3 Pin 50
    assign            XIO_6[6] = RGB_LED_GREEN;//UB6 -- J8 Pin 7 -- U3 Pin 49
    assign            XIO_6[7] = UB40;//UB40 -- J8 Pin 8 -- U3 Pin 46

    assign            XIO_7[0] = UB43;//UB43 -- J7 Pin 1 -- U3 Pin 28
    assign            XIO_7[1] = UB42;//UB42 -- J7 Pin 2 -- U3 Pin 31
    assign            XIO_7[2] = USER_LEDS[1];//UB1 -- J7 Pin 3 -- U3 Pin 32
    assign            XIO_7[3] = USER_LEDS[2];//UB2 -- J7 Pin 4 -- U3 Pin 33
    assign            XIO_7[4] = UB41;//UB41 -- J7 Pin 5 -- U3 Pin 39
    assign            XIO_7[5] = USER_LEDS[3];//UB3 -- J7 Pin 6 -- U3 Pin 42
    assign            XIO_7[6] = RGB_LED_RED;//UB4 -- J7 Pin 7 -- U3 Pin 43
    assign            XIO_7[7] = RGB_LED_BLUE;//UB5 -- J7 Pin 8 -- U3 Pin 44

    assign            XIO_8[0] = UB45;//UB45 -- J13 Pin 1 -- U3 Pin 7
    assign            XIO_8[1] = USER_LEDS[0];//UB0 -- J13 Pin 2 -- U3 Pin 10
    assign            XIO_8[2] = UB44;//UB44 -- J13 Pin 3 -- U3 Pin 11
    assign            XIO_8[3] = LED_DISPLAY_2[7];// -- UB38 -- J13 Pin 4 -- U3 Pin 69
    assign            XIO_8[4] = USER_LEDS[7];//UB22 -- J13 Pin 5 -- U3 Pin 106
    assign            XIO_8[5] = UB9;//UB9 -- J13 Pin 6 -- U3 Pin 23
    assign            XIO_8[6] = UB58;//UB58 -- J13 Pin 7 -- U3 Pin 24
    assign            XIO_8[7] = UB59;//UB59 -- J13 Pin 8 -- U3 Pin 25
   
   
   //**************************************************
//* 	Reset Signal	
//**************************************************
	always @(posedge CLK_50MHZ or negedge RST_N)
	begin
	  if(!RST_N)
	  begin
			reset_signal_reg <= 1'b0;
			reset_counter <= 0;
	  end
	  else
	  begin
		if( reset_counter < GLOBAL_RESET_COUNT )
		begin
			reset_signal_reg <= 1'b0;
			reset_counter <= reset_counter + 1'b1;
		end
		else
		begin
			reset_signal_reg <= 1'b1;
		end
	  end
	end

	
   //-----------------------------------------------
   // Trigger Out Byte. Device can 
   // send the trigger byte to the host.
   //-----------------------------------------------

  always @(posedge CLK_50MHZ or negedge RST)
  begin
     if(!RST)
	     trigger_out <= 0;
	else
	 begin
         // Push Button swtiches, a button press will send 
         // send the trigger byte to the host.
	     if(!PB_SWITCH_1)
		    trigger_out <= PB_SWITCH_1_ASSERT;
	     else if(!PB_SWITCH_2)
		    trigger_out <= PB_SWITCH_2_ASSERT;
	     else
		    trigger_out <= 0;
	 end
  
  end

   //-----------------------------------------------
   // Transfer to Host in Loop Back Mode 
   //-----------------------------------------------
  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	begin
			 transfer_out_reg <= 1'b0;
			 transfer_to_host_counter <= 0;
			 transfer_to_host_state <= TRANSFER_HOST_IDLE;
	end 
	else 
	begin
			     transfer_out_reg <= 1'b0;
			case(transfer_to_host_state)
             TRANSFER_HOST_IDLE:
			     if(transfer_in_received )	
                 begin			 
			         transfer_out_byte = transfer_in_byte;
			         transfer_to_host_state <= TRANSFER_HOST_LOOPBACK_START;
			     end
             TRANSFER_HOST_LOOPBACK_START:
		         if(transfer_to_host_counter < 4'hf)
			     begin
		             transfer_to_host_counter <= transfer_to_host_counter + 1'd1;
			         transfer_to_host_state <= TRANSFER_HOST_LOOPBACK_START;
			     end
				 else
				 begin
			         transfer_out_reg <= 1'b1;
			         transfer_to_host_state <= TRANSFER_HOST_LOOPBACK_COMPLETE;				 
				 end
             TRANSFER_HOST_LOOPBACK_COMPLETE:
			     begin
			         transfer_out_reg <= 1'b0;
					 transfer_to_host_counter <= 0;
			         transfer_to_host_state <= TRANSFER_HOST_IDLE;
				 end
			 default:
			      begin
			          transfer_out_reg <= 1'b0;
			          transfer_to_host_counter <= 0;
			          transfer_to_host_state <= TRANSFER_HOST_IDLE;
				  end
			 endcase
	end
    end

   //-----------------------------------------------
   // Block In write to memory array is driven by block_byte_ready
   //-----------------------------------------------
/*  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	begin
		 fifo_wren_reg <= 1'b0;
		 block_in_data_counter <= 0;
	end
	else 
	begin
	     if(block_in_rcv & block_byte_ready & !fifo_wren_reg)
		 begin
		     fifo_wren_reg <= 1'b1;
		 end
	     else if(block_in_rcv & block_byte_ready & fifo_wren_reg)
		 begin
		     fifo_wren_reg <= 1'b1;
			 mem_array[block_in_data_counter] <= block_in_data;
		 end
	     else if(block_in_rcv & !block_byte_ready & fifo_wren_reg)
		 begin
		     fifo_wren_reg <= 1'b0;
			 block_in_data_counter <= block_in_data_counter + 1'd1;
		 end
	     else if(!block_in_rcv)
		 begin
		     fifo_wren_reg <= 1'b0;
		     block_in_data_counter <= 0;
		 end
	end 
  end
*/   

   //-----------------------------------------------
   // Block Out
   //-----------------------------------------------
/*  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	begin
		state_block_loopback <= BLOCK_LOOPBACK_IDLE;
	end
	else 
	begin
	     if(block_in_loopback) 
		 begin
		     case(state_block_loopback)
			 BLOCK_LOOPBACK_IDLE:
			 begin
			     if(block_out_reg)
				    state_block_loopback <= BLOCK_LOOPBACK_INIT;
			 end
			 BLOCK_LOOPBACK_INIT:
			 begin
				 state_block_loopback <= FIFO_EN_HIGH;
		     end
			 FIFO_EN_HIGH:
		     begin
	             if(block_byte_ready )
				 begin
				     state_block_loopback <= FIFO_EN_LOW;
				 end
				 else
				 begin
				     state_block_loopback <= FIFO_EN_HIGH;
				 end
		     end
			 FIFO_EN_LOW:
		     begin
				 if(block_out_counter == 0)
				 begin
					 if(block_out_reg)
				         state_block_loopback <= BLOCK_LOOPBACK_IDLE;
				 end
	             else if(!block_byte_ready )
				 begin
				     state_block_loopback <= FIFO_EN_HIGH;
				 end
			 end
			 endcase
		 end
		 else
		 begin
		     state_block_loopback <= BLOCK_LOOPBACK_IDLE;
		 end
	end 
  end
*/

   //-----------------------------------------------
   // Detect Block Transfer Out start
   //-----------------------------------------------
/*  always @(block_out_reg or trigger_in_byte or RST)
  begin
     if(!RST)
	     start_block_transfer = 1'b0;
	 else if(block_out_reg)
	     start_block_transfer = 1'b0;
	 else if((trigger_in_byte[7]))
	     start_block_transfer = 1'b1;
  
  end
*/ 

  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	begin
	   start_block_transfer <= 1'b0;
	end
	else
	begin
	    if(block_out_reg)
	        start_block_transfer <= 1'b0;
	    else if((trigger_in_byte[7]))
	        start_block_transfer <= 1'b1;
    end
  end
 
   //-----------------------------------------------
   // Registers to start Block Transfer Out
   //-----------------------------------------------
  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	begin
		 block_out_reg <= 1'b0;
		 start_block_transfer_reg <= 1'b0;
		 block_out_counter <= 0;
		 block_transfer_out_state <= BLK_TRANSFER_OUT_IDLE;
	end
	else
	begin
       case(block_transfer_out_state)
          BLK_TRANSFER_OUT_IDLE:
		  begin
             if(start_block_transfer )
             begin
                start_block_transfer_reg <= 1'b1;
                block_transfer_out_state <= BLK_TRANSFER_OUT_COUNT;
			 end
          end
          BLK_TRANSFER_OUT_COUNT:
          begin
                block_out_reg <= 1'b1;
                if(block_byte_ready)
                begin
                   block_out_counter <= block_out_counter + 1'd1;
                end
                else if(block_out_counter >= CYCLO_GPIO_MAX_NUMBER )
                begin
                   block_out_counter <= 0;
                   block_out_reg <= 1'b0;
                   block_transfer_out_state <= BLK_TRANSFER_OUT_COMPLETE;
                   //block_transfer_out_state <= BLK_TRANSFER_OUT_IDLE;
                end
          end
          BLK_TRANSFER_OUT_COMPLETE:
          begin
             start_block_transfer_reg <= 1'b0;
             block_transfer_out_state <= BLK_TRANSFER_OUT_IDLE;
          end
		  
          default:
          begin
             block_out_counter <= 0;
             block_out_reg <= 1'b0;
             block_transfer_out_state <= BLK_TRANSFER_OUT_IDLE;
          end
       endcase
	end
  end

     //-----------------------------------------------
   // Block In read fifo enable is driven by block_byte_ready
   //-----------------------------------------------
  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	begin
		block_out_byte <= 8'h00;
	end
	else 
	begin
		     case(block_out_counter)
			 0:
			 begin
			     block_out_byte <= XIO_1;
			 end
			 1:
			 begin
				 block_out_byte <= XIO_2;
		     end
			 2:
		     begin
	             block_out_byte <= XIO_3;
		     end
			 3:
		     begin
				 block_out_byte <= XIO_4;
			 end
			 4:
			 begin
			     block_out_byte <= XIO_5;
			 end
			 5:
			 begin
			     block_out_byte <= XIO_6;
			 end
			 6:
		     begin
				 block_out_byte <= XIO_7;
			 end
			 7:
			 begin
			     block_out_byte <= XIO_8;
			 end
			 endcase
	end
  end


   //-----------------------------------------------
   // RGB LED Assert
   //    
   //-----------------------------------------------
  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	begin
       rgb_led_byte <= transfer_in_byte_userled;
       rgb_led_enable <= 1'b0; 
    end	
	else 
	begin
        if(trigger_in_byte[0])
	    begin
	         rgb_led_byte <= transfer_in_byte_userled;
		     rgb_led_enable <= 1'b1;
        end	
		else
		begin
		   rgb_led_enable <= 1'b0;
		end
    end
  end
  
   //-----------------------------------------------
   // Seven Segment LED Display 
   //    
   //-----------------------------------------------
  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	begin
	   seven_segment_led_display_byte_1 <= 8'h0;
	   seven_segment_led_display_enable_1 <= 1'b0;
	   seven_segment_led_display_byte_2 <= 8'h0;
	   seven_segment_led_display_enable_2 <= 1'b0;
	   seven_segment_led_display_byte_3 <= 8'h0;
	   seven_segment_led_display_enable_3 <= 1'b0;
	   transfer_in_hold_byte <= 8'h0;
	end
	else 
	begin
        if(transfer_in_received_led_display)
	    begin
           transfer_in_hold_byte <= transfer_in_led_display;       
        end	


        if(trigger_in_byte[1])
	    begin
		   seven_segment_led_display_byte_1 <= transfer_in_hold_byte;
		   seven_segment_led_display_enable_1 <= 1'b1;
        end	
		else if(trigger_in_byte[2])
	    begin
		   seven_segment_led_display_byte_2 <= transfer_in_hold_byte;
		   seven_segment_led_display_enable_2 <= 1'b1;
        end	
        else if(trigger_in_byte[3])
	    begin
		   seven_segment_led_display_byte_3 <= transfer_in_hold_byte;
		   seven_segment_led_display_enable_3 <= 1'b1;
        end	
		else
		begin
		   seven_segment_led_display_enable_1 <= 1'b0;
		   seven_segment_led_display_enable_2 <= 1'b0;
		   seven_segment_led_display_enable_3 <= 1'b0;
		end
    end
  end
  
   
   //-----------------------------------------------
   // LED Blinky Load Timer Value
   //-----------------------------------------------

  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	     timer_value <= TIMER_LOW_LIMIT;
	else 
	begin
	   timer_value <= TIMER_LOW_LIMIT;
/*	     if(trigger_in_byte[4])
	         timer_value[15:8] <= transfer_in_byte_userled;
	     else if(trigger_in_byte[5])
	         timer_value[23:16] <= transfer_in_byte_userled;
*/	end 
  end

    
   //-----------------------------------------------
   // LED Blinky Load Static Value
   //-----------------------------------------------

  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	     static_load_value <= 1;
	else 
	begin
		 if(control_register[7:4] == STATIC_VALUE)
		 begin
		      
	        if(trigger_in_byte[6])
	           static_load_value <= transfer_in_received_userled;
		 end
	end 
  end

   //-----------------------------------------------
   // LED Blinky start 
   //-----------------------------------------------
  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
         start_blinky <= 1'b1;
	else 
	begin
         start_blinky <= 1'b1;
 	end 
  end
 
   //-----------------------------------------------
   // LED Blinky Delay Timer Counter 
   //-----------------------------------------------
  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	     led_delay_counter <= TIMER_LOW_LIMIT;
	else 
	begin
	     if(state[SELECT_MODE])
		 led_delay_counter <= timer_value;
		 else if(state[WAIT_FOR_TIMER])
	         led_delay_counter <= led_delay_counter - 1'd1;
	end 
  end

   //-----------------------------------------------
   // Select the LED Function from the control
   // register bits 7 to 4   
   //-----------------------------------------------
  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	     led_reg <= 8'h1;
	else 
	begin
        if (state[SELECT_MODE])
	    begin
	         case(control_register[7:4])
		       SHIFT_LEFT:
		          led_reg = shift_count_value;
		       SHIFT_RIGHT:
		          led_reg = shift_count_value;
		       COUNT_UP:
		          led_reg = shift_count_value;
		       COUNT_DOWN:
		          led_reg = shift_count_value;
		       STATIC_VALUE:
		          led_reg = static_load_value;
		       default:
		          led_reg = shift_count_value;
		      endcase 
        end	
    end
  end
  
   //-----------------------------------------------
   // Increment the Shift/Count value from the control
   // register bits 7 to 4   
   //-----------------------------------------------
  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	     shift_count_value <= 8'h1;
	else 
	begin
	    if(trigger_in_byte[6])
		     shift_count_value <= static_load_value;
		else if (state[LOAD_REGISTER])
	    begin
	         case(control_register[7:4])
		     SHIFT_LEFT:
			 begin
		        shift_count_value[0] <= led_reg[7];
		        shift_count_value[7] <= led_reg[6];
		        shift_count_value[6] <= led_reg[5];
		        shift_count_value[5] <= led_reg[4];
		        shift_count_value[4] <= led_reg[3];
		        shift_count_value[3] <= led_reg[2];
		        shift_count_value[2] <= led_reg[1];
		        shift_count_value[1] <= led_reg[0];
		     end
		     SHIFT_RIGHT:
			 begin
		        shift_count_value[7] <= led_reg[0];
		        shift_count_value[6] <= led_reg[7];
		        shift_count_value[5] <= led_reg[6];
		        shift_count_value[4] <= led_reg[5];
		        shift_count_value[3] <= led_reg[4];
		        shift_count_value[2] <= led_reg[3];
		        shift_count_value[1] <= led_reg[2];
		        shift_count_value[0] <= led_reg[1];
		     end
		     COUNT_UP:
		        shift_count_value <= led_reg + 1'd1;
		     COUNT_DOWN:
		        shift_count_value <= led_reg - 1'd1;
			 STATIC_VALUE:
			    shift_count_value <= led_reg;
		     default:
		     //SHIFT_LEFT:
			 begin
		        shift_count_value[0] <= led_reg[7];
		        shift_count_value[7] <= led_reg[6];
		        shift_count_value[6] <= led_reg[5];
		        shift_count_value[5] <= led_reg[4];
		        shift_count_value[4] <= led_reg[3];
		        shift_count_value[3] <= led_reg[2];
		        shift_count_value[2] <= led_reg[1];
		        shift_count_value[1] <= led_reg[0];
		     end
		    endcase   
         end
	end 
  end

   //-----------------------------------------------
   // Set the USER_LEDS outputs 
   //-----------------------------------------------
  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	     USER_LEDS <= 8'hz;
	else 
	begin
	  if(led_reset)
	     USER_LEDS <= 8'hz;
	  else if(state[LOAD_LEDS])
	  begin
	    if ( led_reg[0] )
			USER_LEDS[0] = 1'b0;
		else 
			USER_LEDS[0] = 1'bz;
		if ( led_reg[1] )
			USER_LEDS[1] = 1'b0;
		else 
			USER_LEDS[1] = 1'bz;
 		if ( led_reg[2] )
			USER_LEDS[2] = 1'b0;
		else 
			USER_LEDS[2] = 1'bz;
		if ( led_reg[3] )
			USER_LEDS[3] = 1'b0;
		else 
			USER_LEDS[3] = 1'bz;
			
		if ( led_reg[4] )
			USER_LEDS[4] = 1'b0;
		else 
			USER_LEDS[4] = 1'bz;
		if ( led_reg[5] )
			USER_LEDS[5] = 1'b0;
		else 
			USER_LEDS[5] = 1'bz;
 		if ( led_reg[6] )
			USER_LEDS[6] = 1'b0;
		else 
			USER_LEDS[6] = 1'bz;
		if ( led_reg[7] )
			USER_LEDS[7] = 1'b0;
		else 
			USER_LEDS[7] = 1'bz;
	  end
	end
  end
  
  
   //-----------------------------------------------
   // State Machine: Send Pattern to LEDs 
   //-----------------------------------------------
   // Next State Logic
   always @(posedge CLK_50MHZ or negedge RST)
     begin
	if (!RST)
	  begin
	     state <= 5'h00;
	     state[IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end

     // State Definitions
   always @ ( state or  start_blinky or
		      led_reg or led_delay_counter or 
			  timer_value or shift_count_value) 
     begin
	     next = 5'h00;

	if (state[IDLE])
	  begin
	     if (start_blinky)
	       next[SELECT_MODE] = 1'b1;
	     else
	       next[IDLE] = 1'b1;
	  end
	
	if (state[SELECT_MODE])
	begin
       next[LOAD_REGISTER] = 1'b1;
	end

	if (state[LOAD_REGISTER])
	begin
        next[LOAD_LEDS] = 1'b1;
    end

	if (state[LOAD_LEDS])
	begin
        next[WAIT_FOR_TIMER] = 1'b1;
    end
			
	if (state[WAIT_FOR_TIMER])
		if (led_delay_counter <= timer_value)
			next[WAIT_FOR_TIMER] = 1'b1;
		else 
			next[SELECT_MODE] = 1'b1;

			
`ifdef SIM
 	   if ( state == ( 1 << IDLE ))
		   state_name = "IDLE";
	   else if ( state == ( 1 << SELECT_MODE ))
		   state_name = "SELECT_MODE";
	   else if ( state == ( 1 << LOAD_REGISTER ))
		   state_name = "LOAD_REGISTER";
	   else if ( state == ( 1 << LOAD_LEDS ))
		   state_name = "LOAD_LEDS";
	   else if ( state == ( 1 << WAIT_FOR_TIMER ))
		   state_name = "WAIT_FOR_TIMER";
`endif


	end//end of state machine

   //-----------------------------------------------
   // Instantiate the Seven Segment LED Display
   /*
   */
   //-----------------------------------------------

	seven_segment_led_display  LED_DISPLAY_INST
	(	
	 .CLK_50MHZ                (CLK_50MHZ),
     .RST                      (RST),

	 .LED_DISPLAY_BYTE_1       (seven_segment_led_display_byte_1),
	 .LED_DISPLAY_ENABLE_1     (seven_segment_led_display_enable_1),
	
	 .LED_DISPLAY_BYTE_2       (seven_segment_led_display_byte_2),
	 .LED_DISPLAY_ENABLE_2     (seven_segment_led_display_enable_2), 
	
	 .LED_DISPLAY_BYTE_3       (seven_segment_led_display_byte_3),
	 .LED_DISPLAY_ENABLE_3     (seven_segment_led_display_enable_3),
	 
	 .SEVEN_SEGMENT_LED_1      (LED_DISPLAY_1),
	 .SEVEN_SEGMENT_LED_2      (LED_DISPLAY_2),
	 .SEVEN_SEGMENT_LED_3      (LED_DISPLAY_3),
	 
	 .LED_DISPLAY_CONTROL      (control_register)
	);

   //-----------------------------------------------
   // Instantiate the Seven Segment LED Display
   /*
       RGB_LED_BLUE <= 1'b1;
       RGB_LED_RED <= 1'b1;
       RGB_LED_GREEN <= 1'b1;
   */
   //-----------------------------------------------

	rgb_led_control            RGB_LED_INST
	(	
	 .CLK_50MHZ                (CLK_50MHZ),
     .RST                      (RST),

     .RGB_LED_BLUE             (RGB_LED_BLUE),
     .RGB_LED_RED              (RGB_LED_RED),
     .RGB_LED_GREEN            (RGB_LED_GREEN),

     .LED_RGB_BYTE             (rgb_led_byte),
	 .LED_RGB_ENABLE           (rgb_led_enable),


     .LED_DISPLAY_CONTROL      (control_register)
	);
   
   //-----------------------------------------------
   // Control Register from Transfer In 
   //-----------------------------------------------
  /*
     Control Register Register Map
	 control_register     = 8'h50  =  USER_LEDS COUNT_DOWN
	 control_register     = 8'h40  =  USER_LEDS COUNT_UP 
	 control_register     = 8'h30  =  USER_LEDS SHIFT_RIGHT
	 control_register     = 8'h20  =  USER_LEDS SHIFT_LEFT
	 control_register     = 8'h08  =  LED_DISPLAY STOP_DISPLAY
	 control_register     = 8'h07  =  LED_DISPLAY START_DISPLAY
	 control_register     = 8'h06  =  LED_DISPLAY STATIC_VALUE
	 control_register     = 8'h05  =  LED_DISPLAY COUNT_DOWN
	 control_register     = 8'h04  =  LED_DISPLAY COUNT_UP
	 control_register     = 8'h03  =  LED_DISPLAY SHIFT_RIGHT
	 control_register     = 8'h02  =  LED_DISPLAY SHIFT_LEFT
	 

  */

  active_control_register      ACTIVE_CONTROL_REG_INST
  (
   .CLK                        (CLK_50MHZ),
   .RST                        (RST),
   .TRANSFER_IN_RECEIVED       (transfer_in_received),
   .TRANSFER_IN_BYTE           (transfer_in_byte),

   .CONTROL_REGISTER           (control_register)
   );
   
   //-----------------------------------------------
   // Instantiate the EPT Library
   //-----------------------------------------------

	active_spi_library	   ACTIVE_SPI_LIBRARY_INST
	(	
	.aa                        (aa),
	.SCLK                      (SPI_SCLK),
	.MISO                      (SPI_MISO),
	.MOSI                      (spi_mosi),
	.MIOSIO_1                  (SPI_MIOSIO_1),
	.SS                        (SPI_SS),

	.UC_IN                     (UC_IN),
	.UC_OUT                    (UC_OUT),
	
	.STATE_OUT                 (active_serial_state_out),
	.TEST_OUT                  (active_transfer_test_out) 
	
	);

	
   //-----------------------------------------------
   // Instantiate the EPT Library
   //-----------------------------------------------
wire [22*5-1:0]  uc_out_m;
eptWireOR # (.N(5)) wireOR (UC_OUT, uc_out_m);


 /*   Active Trigger Register Mapping
	
	trigger_in_byte[0]  =  Load RGB LED Value
	trigger_in_byte[1]  =  Load 7 Segment LED Display 1
	trigger_in_byte[2]  =  Load 7 Segment LED Display 2
	trigger_in_byte[3]  =  Load 7 Segment LED Display 3
	trigger_in_byte[4]  =  Load Timer Value, timer_value[15:8]
	trigger_in_byte[5]  =  Load Timer Value, timer_value[23:16]
	trigger_in_byte[6]  =  LED Function Load
	trigger_in_byte[7]  =  start_block_transfer = 1'b1
*/	


	active_trigger             ACTIVE_TRIGGER_INST
	(
	 .uc_clk                   (CLK_50MHZ),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 0*22 +: 22 ]),

	 .trigger_to_host          (trigger_out),
	 .trigger_to_device        (trigger_in_byte)
	
	);

    active_transfer            ACTIVE_TRANSFER_INST_1
	(
	 .uc_clk                   (CLK_50MHZ),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 1*22 +: 22 ]),
	
	 .start_transfer           (start_transfer_userled),
	 .transfer_received        (transfer_in_received_userled),
	 
	 .transfer_busy            (),
	
	 .uc_addr                  (3'h1),

	 .transfer_to_host         (transfer_out_byte_userled),
	 .transfer_to_device       (transfer_in_byte_userled)	
    );
	
	active_transfer            ACTIVE_TRANSFER_INST_2
	(
	 .uc_clk                   (CLK_50MHZ),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 2*22 +: 22 ]),
	
	 .start_transfer           (transfer_out_reg),
	 .transfer_received        (transfer_in_received),
	
	 .transfer_busy            (),
	
	 .uc_addr                  (3'h2),

	 .transfer_to_host         (transfer_out_byte),
	 .transfer_to_device       (transfer_in_byte)	
	);
	
	active_transfer            ACTIVE_TRANSFER_INST_3
	(
	 .uc_clk                   (CLK_50MHZ),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 3*22 +: 22 ]),
	
	 .start_transfer           (transfer_out_led_display_start),
	 .transfer_received        (transfer_in_received_led_display),
	
	 .transfer_busy            (),
	
	 .uc_addr                  (3'h3),

	 .transfer_to_host         (transfer_out_led_display),
	 .transfer_to_device       (transfer_in_led_display)	
	);
	
	
	active_block               BLOCK_TRANSFER_INST
	(
	 .uc_clk                   (CLK_50MHZ),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 4*22 +: 22 ]),
	
	 .start_transfer           (block_out_reg),
	 .transfer_received        (block_in_rcv),
	 
	 .transfer_ready           (block_byte_ready),
	 .transfer_busy            (block_busy),

	 .ept_length               (ept_length),
	
	 .uc_addr                  (3'h4),
	 .uc_length                (CYCLO_GPIO_MAX_NUMBER),

	 .transfer_to_host         (block_out_byte),
	 .transfer_to_device       (block_in_data)
	
	);

	
endmodule



