//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2021
//#
//#
//# File Name:  cycloprologic_define.v
//# Author:     Earth People Technology
//# Date:       May 31, 2021
//# Revision:   A
//#
//# Development: EPT CycloProLogic Demo Project 
//# Application: Altera Cyclone IV FPGA
//# Description: This file contains verilog code which will allow access
//#              to Active Transfer Library. 
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			5/31/21	1			Created
//#
//#									
//#
//#######################################################################
`ifndef _TOP_CYCLOPROLOGIC_DEFINE
 `define _TOP_CYCLOPROLOGIC_DEFINE

 
    //////////////////////////////////////////////////////
    //  CYCLOPROLOGIC MEMORY MAP
    //////////////////////////////////////////////////////
	//ADDRESS                   FUNCTION              RANGE
	//0x00                      
	//0x01                      
	//0x02                      
	//0x03                      I2C_0_DATA1_MEMORY_LOCATION
	//0x04                      I2C_0_DATA2_MEMORY_LOCATION
	//0x05                      I2C_0_DATA3_MEMORY_LOCATION
	//0x06                      I2C_0_DATA4_MEMORY_LOCATION
	//0x07                      I2C_1_DATA1_MEMORY_LOCATION
	//0x08                      I2C_1_DATA2_MEMORY_LOCATION
	//0x09                      I2C_1_DATA3_MEMORY_LOCATION
	//0x0a                      I2C_1_DATA4_MEMORY_LOCATION
	//0x0b                      Reserved
	// To                       Reserved
	//0x0f                      Reserved
    //0x10                      
    //0x12                      
    //0x14                      
    //0x16                      
	//0x17                      Reserved
	// To                       Reserved
	//0x19                      Reserved
    //0x1a                      
    //0x1b                      
	// To                       Reserved
	//0xff                      Reserved
	
	
    //////////////////////////////////////////////////////
    //  CYCLOPROLOGIC MEMORY LOCATIONS
    //////////////////////////////////////////////////////

	//Memory Location for I2C_0 Data Information
    `define I2C_0_WRITE_DATA1_MEMORY_LOCATION        8'h03
    `define I2C_0_WRITE_DATA2_MEMORY_LOCATION        8'h04
    `define I2C_0_READ_DATA1_MEMORY_LOCATION         8'h05
    `define I2C_0_READ_DATA2_MEMORY_LOCATION         8'h06
	
	//Memory Location for I2C_1 Data Information
    `define I2C_1_WRITE_DATA1_MEMORY_LOCATION        8'h07
    `define I2C_1_WRITE_DATA2_MEMORY_LOCATION        8'h08
    `define I2C_1_READ_DATA1_MEMORY_LOCATION         8'h09
    `define I2C_1_READ_DATA2_MEMORY_LOCATION         8'h0a
	

    //////////////////////////////////////////////////////
    //  CYCLOPROLOGIC CONTROL REGISTERS
    //////////////////////////////////////////////////////
	//ADDRESS                   FUNCTION              BITS
    //0x0                       Reserved              0 - 
	//                                                1 - 
	//                                                2 - 
	//                                                3 - 
	//                                                4 - 
	//                                                5 - 
	//                                                6 - 
	//                                                7 - 
    //0x01                      Reserved              0 - 
	//                                                1 - 
	//                                                2 - 
	//                                                3 -
	//                                                4 - 
	//                                                5 - 
	//                                                6 - 
	//                                                7 - 
    //0x02                      I2C - 0 Control       0 - 
	//                                                1 - 
	//                                                2 - 
	//                                                3 - 
	//                                                4 - 
	//                                                5 - 
	//                                                6 - 
	//                                                7 - 
    //0x03                      I2C - 1 Control       0 - 
	//                                                1 - 
	//                                                2 - 
	//                                                3 - 
	//                                                4 - 
	//                                                5 - 
	//                                                6 - 
	//                                                7 - 
    //0x04                     Reserved               0 - 
	//                                                1 - 
	//                                                2 - 
	//                                                3 - 
	//                                                4 - 
	//                                                5 - 
	//                                                6 - 
	//                                                7 - 
    //0x05                    Reset Control           0 - 
	//                                                1 - 
	//                                                2 - 
	//                                                3 - 
	//                                                4 - 
	//                                                5 - 
	//                                                6 - 
	//                                                7 - 

    //////////////////////////////////////////////////////
    //  CYCLOPROLOGIC  CONTROL REGISTER Bits
    //////////////////////////////////////////////////////

    //Memory Read Cycle, Wait States for MEMORY_RD_DATA
    `define MEMORY_RD_LATCH_WAIT_STATE	          8'h05
    `define MEMORY_RD_COMPLETE_WAIT_STATE	      8'h0a

    //////////////////////////////////////////////////////
    //  CYCLOPROLOGIC  Commands
    //////////////////////////////////////////////////////

    //I2C Bus 0 Commands
    `define I2C_WRITE_COMMAND                     1'h0
    `define I2C_READ_COMMAND                      1'h1
	
	//AMS1115 Commands
    `define AMS1115_CHIP_1_FIXED_ADDRESS          8'h00 //AMS115 sets pre-defined address as 00
    `define AMS1115_CHIP_2_FIXED_ADDRESS          8'h00 //

   //Memory Read Wait States
    `define MEMORY_RD_WAIT_STATES                 8'h03 
	
	
	`endif //  `ifndef _TOP_DEFINE

